@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile;
 use vars '$VERSION';
-$VERSION = '0.72';
+$VERSION = '0.73';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use XML::LibXML;
@@ -31,7 +31,8 @@ __PACKAGE__->addSchemaDirs(__FILE__);
 
 
 sub new($@)
-{   my ($class, $top) = (shift, shift);
+{   my $class = shift;
+    my $top   = @_ % 2 ? shift : undef;
 
     $class ne __PACKAGE__
        or panic "you should instantiate a sub-class, $class is base only";
@@ -97,37 +98,29 @@ sub dataToXML($)
     defined $thing
         or return;
 
-    my ($xml, $source, %details);
+    my ($xml, %details);
     if(ref $thing && UNIVERSAL::isa($thing, 'XML::LibXML::Node'))
-    {   $xml    = $thing;
-        $source = ref $thing;
+    {   ($xml, %details) = $self->_parsedNode($thing);
     }
     elsif(ref $thing eq 'SCALAR')   # XML string as ref
-    {   $xml    = $self->_parse($thing);
-        $source = ref $thing;
+    {   ($xml, %details) = $self->_parseScalar($thing);
     }
     elsif(ref $thing eq 'GLOB')     # from file-handle
-    {   $xml    = $parser->parse_fh($thing);
-        $xml    = $xml->documentElement if defined $xml;
-        $source = ref $thing;
+    {   ($xml, %details) = $self->_parseFileHandle($thing);
     }
     elsif($thing =~ m/^\s*\</)      # XML starts with '<', rare for files
-    {   $xml    = $self->_parse(\$thing);
-        $source = 'string';
+    {   ($xml, %details) = $self->_parseScalar(\$thing);
     }
     elsif(my $known = $self->knownNamespace($thing))
     {   my $fn  = $self->findSchemaFile($known)
             or error __x"cannot find pre-installed name-space file named {path} for {name}"
                  , path => $known, name => $thing;
 
-        $xml    = $self->_parseFile($fn);
-        $source = "known namespace $thing";
-        $details{filename} = $fn;
+        ($xml, %details) = $self->_parseFile($fn);
+        $details{source} = "known namespace $thing";
     }
     elsif(-f $thing)
-    {   $xml    = $self->_parseFile($thing);
-        $source = "file";
-        $details{filename} = $thing;
+    {   ($xml, %details) = $self->_parseFile($thing);
     }
     else
     {   my $data = "$thing";
@@ -139,16 +132,44 @@ sub dataToXML($)
     wantarray ? ($xml, %details) : $xml;
 }
 
-sub _parse($)
+sub _parsedNode($)
+{   my ($thing, $node) = @_;
+    trace "using preparsed XML node";
+
+    ( $node
+    , source => ref $node
+    );
+}
+
+sub _parseScalar($)
 {   my ($thing, $data) = @_;
+    trace "parsing XML from string $data";
     my $xml = $parser->parse_string($$data);
-    defined $xml ? $xml->documentElement : undef;
+
+    ( (defined $xml ? $xml->documentElement : undef)
+    , source => ref $data
+    );
 }
 
 sub _parseFile($)
 {   my ($thing, $fn) = @_;
+    trace "parsing XML from file $fn";
     my $xml = $parser->parse_file($fn);
-    defined $xml ? $xml->documentElement : undef;
+
+    ( (defined $xml ? $xml->documentElement : undef)
+    , source   => 'file'
+    , filename => $fn
+    );
+}
+
+sub _parseFileHandle($)
+{   my ($thing, $fh) = @_;
+    trace "parsing XML from open file $fh";
+    my $xml = $parser->parse_fh($fh);
+
+    ( (defined $xml ? $xml->documentElement : undef)
+    , source => ref $thing
+    );
 }
 
 
