@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::Translate;
 use vars '$VERSION';
-$VERSION = '0.72';
+$VERSION = '0.73';
 
 # Errors are either in class 'usage': called with request
 #                         or 'schema': syntax error in schema
@@ -17,9 +17,9 @@ use List::Util  'first';
 
 use XML::Compile::Schema::Specs;
 use XML::Compile::Schema::BuiltInFacets;
-use XML::Compile::Schema::BuiltInTypes   qw/%builtin_types/;
-use XML::Compile::Util                   qw/pack_type unpack_type/;
-use XML::Compile::Iterator               ();
+use XML::Compile::Schema::BuiltInTypes qw/%builtin_types/;
+use XML::Compile::Util                 qw/pack_type unpack_type type_of_node/;
+use XML::Compile::Iterator             ();
 
 # Elements from the schema to ignore: remember, we are collecting data
 # from the schema, but only use selective items to produce processors.
@@ -633,6 +633,26 @@ sub particleBlock($)
     ($label => $self->make($blocktype => $tree->path, @pairs));
 }
 
+sub findSgMemberNodes($)
+{   my ($self, $type) = @_;
+    my @subgrps;
+    foreach my $subgrp ($self->namespaces->findSgMembers($type))
+    {   my $node = $subgrp->{node};
+        push @subgrps, $node;
+
+        my $abstract = $node->getAttribute('abstract') || 'false';
+        $self->isTrue($abstract) or next;
+
+        my $groupname = $node->getAttribute('name')
+            or error __x"substitutionGroup element needs name at {where}"
+                 , where => $node->path, class => 'schema';
+
+        my $subtype   = pack_type $self->{tns}, $groupname;
+        push @subgrps, $self->findSgMemberNodes($subtype);
+    }
+    @subgrps;
+}
+        
 sub particleElementSubst($)
 {   my ($self, $tree) = @_;
 
@@ -647,8 +667,7 @@ sub particleElementSubst($)
  
     my $tns     = $self->{tns};
     my $type    = pack_type $tns, $groupname;
-    my @subgrps = map {$_->{node}}
-        $self->namespaces->findSgMembers($type);
+    my @subgrps = $self->findSgMemberNodes($type);
 
     # at least the base is expected
     @subgrps
